@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.test.test_internet -*-
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 
@@ -28,7 +28,7 @@ from twisted.internet.interfaces import IResolverSimple, IReactorPluggableResolv
 from twisted.internet.interfaces import IConnector, IDelayedCall
 from twisted.internet import main, error, abstract, defer, threads
 from twisted.python import log, failure, reflect
-from twisted.python.runtime import seconds as runtimeSeconds, platform
+from twisted.python.runtime import seconds as runtimeSeconds, platform, platformType
 from twisted.internet.defer import Deferred, DeferredList
 from twisted.persisted import styles
 
@@ -393,10 +393,15 @@ class _ThreePhaseEvent(object):
 class ReactorBase(object):
     """
     Default base class for Reactors.
+
+    @type _stopped: C{bool}
+    @ivar _stopped: A flag which is true between paired calls to C{reactor.run}
+        and C{reactor.stop}.
     """
 
     implements(IReactorCore, IReactorTime, IReactorPluggableResolver)
 
+    _stopped = True
     installed = False
     usingThreads = False
     resolver = BlockingResolver()
@@ -485,10 +490,11 @@ class ReactorBase(object):
         """
         See twisted.internet.interfaces.IReactorCore.stop.
         """
-        if not self.running:
+        if self._stopped:
             raise error.ReactorNotRunning(
                 "Can't stop reactor that isn't running.")
-        self.fireSystemEvent("shutdown")
+        self._stopped = True
+        self.callLater(0, self.fireSystemEvent, "shutdown")
 
     def crash(self):
         """
@@ -579,6 +585,7 @@ class ReactorBase(object):
                     "since Twisted 8.0",
                     category=DeprecationWarning, stacklevel=3)
         self.running = True
+        self._stopped = False
         threadable.registerAsIOThread()
         self.fireSystemEvent('startup')
 
@@ -967,4 +974,96 @@ class BasePort(abstract.FileDescriptor):
         raise RuntimeError, "doWrite called on a %s" % reflect.qual(self.__class__)
 
 
+
+class _SignalReactorMixin:
+    """
+    Private mixin to manage signals: it installs signal handlers at start time,
+    and define run method.
+
+    It can only be used mixed in with L{ReactorBase}, and has to be defined
+    first in the inheritance (so that method resolution order finds
+    startRunning first).
+    """
+
+    def _handleSignals(self):
+        """
+        Install the signal handlers for the Twisted event loop.
+        """
+        try:
+            import signal
+        except ImportError:
+            log.msg("Warning: signal module unavailable -- "
+                    "not installing signal handlers.")
+            return
+
+        if signal.getsignal(signal.SIGINT) == signal.default_int_handler:
+            # only handle if there isn't already a handler, e.g. for Pdb.
+            signal.signal(signal.SIGINT, self.sigInt)
+        signal.signal(signal.SIGTERM, self.sigTerm)
+
+        # Catch Ctrl-Break in windows
+        if hasattr(signal, "SIGBREAK"):
+            signal.signal(signal.SIGBREAK, self.sigBreak)
+
+        if platformType == 'posix':
+            signal.signal(signal.SIGCHLD, self._handleSigchld)
+
+
+    def _handleSigchld(self, signum, frame, _threadSupport=platform.supportsThreads()):
+        """
+        Reap all processes on SIGCHLD.
+
+        This gets called on SIGCHLD. We do no processing inside a signal
+        handler, as the calls we make here could occur between any two
+        python bytecode instructions. Deferring processing to the next
+        eventloop round prevents us from violating the state constraints
+        of arbitrary classes.
+        """
+        from twisted.internet.process import reapAllProcesses
+        if _threadSupport:
+            self.callFromThread(reapAllProcesses)
+        else:
+            self.callLater(0, reapAllProcesses)
+
+
+    def startRunning(self, installSignalHandlers=True):
+        """
+        Forward call to ReactorBase, arrange for signal handlers to be
+        installed if asked.
+        """
+        if installSignalHandlers:
+            # Make sure this happens before after-startup events, since the
+            # expectation of after-startup is that the reactor is fully
+            # initialized.  Don't do it right away for historical reasons
+            # (perhaps some before-startup triggers don't want there to be a
+            # custom SIGCHLD handler so that they can run child processes with
+            # some blocking api).
+            self.addSystemEventTrigger(
+                'during', 'startup', self._handleSignals)
+        ReactorBase.startRunning(self)
+
+
+    def run(self, installSignalHandlers=True):
+        self.startRunning(installSignalHandlers=installSignalHandlers)
+        self.mainLoop()
+
+
+    def mainLoop(self):
+        while self.running:
+            try:
+                while self.running:
+                    # Advance simulation time in delayed event
+                    # processors.
+                    self.runUntilCurrent()
+                    t2 = self.timeout()
+                    t = self.running and t2
+                    self.doIteration(t)
+            except:
+                log.msg("Unexpected error in main loop.")
+                log.err()
+            else:
+                log.msg('Main loop terminated.')
+
+
+
 __all__ = []
