@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 
@@ -171,7 +171,15 @@ def getConnectError(e):
     return klass(number, string)
 
 
-class ConnectionLost(Exception):
+
+class ConnectionClosed(Exception):
+    """
+    Connection was closed, whether cleanly or non-cleanly.
+    """
+
+
+
+class ConnectionLost(ConnectionClosed):
     """Connection to the other side was lost in a non-clean fashion"""
 
     def __str__(self):
@@ -182,7 +190,8 @@ class ConnectionLost(Exception):
         return s
 
 
-class ConnectionDone(Exception):
+
+class ConnectionDone(ConnectionClosed):
     """Connection was closed cleanly"""
 
     def __str__(self):
@@ -218,6 +227,23 @@ class AlreadyCancelled(ValueError):
         s = '%s.' % s
         return s
 
+
+
+class PotentialZombieWarning(Warning):
+    """
+    Emitted when L{IReactorProcess.spawnProcess} is called in a way which may
+    result in termination of the created child process not being reported.
+    """
+    MESSAGE = (
+        "spawnProcess called, but the SIGCHLD handler is not "
+        "installed. This probably means you have not yet "
+        "called reactor.run, or called "
+        "reactor.run(installSignalHandler=0). You will probably "
+        "never see this process finish, and it may become a "
+        "zombie process.")
+
+
+
 class ProcessDone(ConnectionDone):
     """A process has ended without apparent errors"""
 
