@@ -229,7 +229,7 @@ class DocBuilder(LoreBuilderMixin):
     Generate HTML documentation for projects.
     """
 
-    def build(self, version, resourceDir, docDir, template, deleteInput=False):
+    def build(self, version, resourceDir, docDir, template, apiBaseURL=None, deleteInput=False):
         """
         Build the documentation in C{docDir} with Lore.
 
@@ -249,6 +249,12 @@ class DocBuilder(LoreBuilderMixin):
         @param template: The template used to generate the documentation.
         @type template: L{twisted.python.filepath.FilePath}
 
+        @type apiBaseURL: C{str} or C{NoneType}
+        @param apiBaseURL: A format string which will be interpolated with the
+            fully-qualified Python name for each API link.  For example, to
+            generate the Twisted 8.0.0 documentation, pass
+            C{"http://twistedmatrix.com/documents/8.0.0/api/%s.html"}.
+
         @param deleteInput: If True, the input documents will be deleted after
             their output is generated.
         @type deleteInput: C{bool}
@@ -261,10 +267,14 @@ class DocBuilder(LoreBuilderMixin):
         filenames = [x.path for x in inputFiles]
         if not filenames:
             raise NoDocumentsFound("No input documents found in %s" % (docDir,))
-        arguments = ["--config", "template=%s" % (template.path,),
-                     "--config", "ext=.html",
-                     "--config", "version=%s" % (version,),
-                     "--linkrel", linkrel] + filenames
+        if apiBaseURL is not None:
+            arguments = ["--config", "baseurl=" + apiBaseURL]
+        else:
+            arguments = []
+        arguments.extend(["--config", "template=%s" % (template.path,),
+                          "--config", "ext=.html",
+                          "--config", "version=%s" % (version,),
+                          "--linkrel", linkrel] + filenames)
         self.lore(arguments)
         if deleteInput:
             for inputFile in inputFiles:
@@ -323,9 +333,73 @@ class ManBuilder(LoreBuilderMixin):
 
 
 
+class APIBuilder(object):
+    """
+    Generate API documentation from source files using
+    U{pydoctor<http://codespeak.net/~mwh/pydoctor/>}.  This requires
+    pydoctor to be installed and usable (which means you won't be able to
+    use it with Python 2.3).
+    """
+    def build(self, projectName, projectURL, sourceURL, packagePath, outputPath):
+        """
+        Call pydoctor's entry point with options which will generate HTML
+        documentation for the specified package's API.
+
+        @type projectName: C{str}
+        @param projectName: The name of the package for which to generate
+            documentation.
+
+        @type projectURL: C{str}
+        @param projectURL: The location (probably an HTTP URL) of the project
+            on the web.
+
+        @type sourceURL: C{str}
+        @param sourceURL: The location (probably an HTTP URL) of the root of
+            the source browser for the project.
+
+        @type packagePath: L{FilePath}
+        @param packagePath: The path to the top-level of the package named by
+            C{projectName}.
+
+        @type outputPath: L{FilePath}
+        @param outputPath: An existing directory to which the generated API
+            documentation will be written.
+        """
+        from pydoctor.driver import main
+        main(
+            ["--project-name", projectName,
+             "--project-url", projectURL,
+             "--system-class", "pydoctor.twistedmodel.TwistedSystem",
+             "--project-base-dir", packagePath.parent().path,
+             "--html-viewsource-base", sourceURL,
+             "--add-package", packagePath.path,
+             "--html-output", outputPath.path,
+             "--quiet", "--make-html"])
+
+
+
 class BookBuilder(LoreBuilderMixin):
     """
     Generate the LaTeX and PDF documentation.
+
+    The book is built by assembling a number of LaTeX documents.  Only the
+    overall document which describes how to assemble the documents is stored
+    in LaTeX in the source.  The rest of the documentation is generated from
+    Lore input files.  These are primarily XHTML files (of the particular
+    Lore subset), but man pages are stored in GROFF format.  BookBuilder
+    expects all of its input to be Lore XHTML format, so L{ManBuilder}
+    should be invoked first if the man pages are to be included in the
+    result (this is determined by the book LaTeX definition file).
+    Therefore, a sample usage of BookBuilder may look something like this:
+
+        man = ManBuilder()
+        man.build(FilePath("doc/core/man"))
+        book = BookBuilder()
+        book.build(
+            FilePath('doc/core/howto'),
+            [FilePath('doc/core/howto'), FilePath('doc/core/howto/tutorial'),
+             FilePath('doc/core/man'), FilePath('doc/core/specifications')],
+            FilePath('doc/core/howto/book.tex'), FilePath('/tmp/book.pdf'))
     """
     def run(self, command):
         """
@@ -346,7 +420,7 @@ class BookBuilder(LoreBuilderMixin):
 
     def buildTeX(self, howtoDir):
         """
-        Build LaTex files for lore input files in the given directory.
+        Build LaTeX files for lore input files in the given directory.
 
         Input files ending in .xhtml will be considered. Output will written as
         .tex files.
@@ -485,18 +559,26 @@ class DistributionBuilder(object):
     from twisted.python.dist import twisted_subprojects as subprojects
     blacklist = ["vfs", "web2"]
 
-    def __init__(self, rootDirectory, outputDirectory):
+    def __init__(self, rootDirectory, outputDirectory, apiBaseURL=None):
         """
         Create a distribution builder.
 
         @param rootDirectory: root of a Twisted export which will populate
             subsequent tarballs.
         @type rootDirectory: L{FilePath}.
+
         @param outputDirectory: The directory in which to create the tarballs.
         @type outputDirectory: L{FilePath}
+
+        @type apiBaseURL: C{str} or C{NoneType}
+        @param apiBaseURL: A format string which will be interpolated with the
+            fully-qualified Python name for each API link.  For example, to
+            generate the Twisted 8.0.0 documentation, pass
+            C{"http://twistedmatrix.com/documents/8.0.0/api/%s.html"}.
         """
         self.rootDirectory = rootDirectory
         self.outputDirectory = outputDirectory
+        self.apiBaseURL = apiBaseURL
         self.manBuilder = ManBuilder()
         self.docBuilder = DocBuilder()
 
@@ -522,7 +604,7 @@ class DistributionBuilder(object):
         if path.isdir():
             try:
                 self.docBuilder.build(version, howtoPath, path,
-                    templatePath, True)
+                    templatePath, self.apiBaseURL, True)
             except NoDocumentsFound:
                 pass
 
