@@ -5,7 +5,7 @@
 Test cases covering L{twisted.python.filepath} and L{twisted.python.zippath}.
 """
 
-import os, time, pickle, errno, zipfile
+import os, time, pickle, errno, zipfile, stat
 
 from twisted.python.win32 import WindowsError, ERROR_DIRECTORY
 from twisted.python import filepath
@@ -257,6 +257,19 @@ class ZipFilePathTestCase(AbstractFilePathTestCase):
 
 class FilePathTestCase(AbstractFilePathTestCase):
 
+    def test_chmod(self):
+        """
+        Make sure that calling L{FilePath.chmod} modifies the permissions of
+        the passed file as expected (using C{os.stat} to check). We use some
+        basic modes that should work everywhere (even on Windows).
+        """
+        for mode in (0555, 0777):
+            self.path.child("sub1").chmod(mode)
+            self.assertEquals(
+                stat.S_IMODE(os.stat(self.path.child("sub1").path).st_mode),
+                mode)
+
+
     def test_getAndSet(self):
         content = 'newcontent'
         self.path.child('new').setContent(content)
@@ -267,17 +280,58 @@ class FilePathTestCase(AbstractFilePathTestCase):
         newcontent = self.path.child('new').getContent()
         self.failUnlessEqual(content, newcontent)
 
-    def testSymbolicLink(self):
+
+    def test_symbolicLink(self):
+        """
+        Verify the behavior of the C{isLink} method against links and
+        non-links. Also check that the symbolic link shares the directory
+        property with its target.
+        """
         s4 = self.path.child("sub4")
         s3 = self.path.child("sub3")
         os.symlink(s3.path, s4.path)
-        self.failUnless(s4.islink())
-        self.failIf(s3.islink())
-        self.failUnless(s4.isdir())
-        self.failUnless(s3.isdir())
+        self.assertTrue(s4.islink())
+        self.assertFalse(s3.islink())
+        self.assertTrue(s4.isdir())
+        self.assertTrue(s3.isdir())
+
+
+    def test_linkTo(self):
+        """
+        Verify that symlink creates a valid symlink that is both a link and a
+        file if its target is a file, or a directory if its target is a
+        directory.
+        """
+        targetLinks = [
+            (self.path.child("sub2"), self.path.child("sub2.link")),
+            (self.path.child("sub2").child("file3.ext1"),
+             self.path.child("file3.ext1.link"))
+            ]
+        for target, link in targetLinks:
+            target.linkTo(link)
+            self.assertTrue(link.islink(), "This is a link")
+            self.assertEquals(target.isdir(), link.isdir())
+            self.assertEquals(target.isfile(), link.isfile())
+
+
+    def test_linkToErrors(self):
+        """
+        Verify C{linkTo} fails in the following case:
+            - the target is in a directory that doesn't exist
+            - the target already exists
+        """
+        self.assertRaises(OSError, self.path.child("file1").linkTo,
+                          self.path.child('nosub').child('file1'))
+        self.assertRaises(OSError, self.path.child("file1").linkTo,
+                          self.path.child('sub1').child('file2'))
+
+
+    if not getattr(os, "symlink", None):
+        skipMsg = "Your platform does not support symbolic links."
+        test_symbolicLink.skip = skipMsg
+        test_linkTo.skip = skipMsg
+        test_linkToErrors.skip = skipMsg
 
-    if not hasattr(os, "symlink"):
-        testSymbolicLink.skip = "Your platform does not support symbolic links."
 
     def testMultiExt(self):
         f3 = self.path.child('sub3').child('file3')
@@ -386,6 +440,23 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.path.remove()
         self.failIf(self.path.exists())
 
+
+    def test_removeWithSymlink(self):
+        """
+        For a path which is a symbolic link, L{FilePath.remove} just deletes
+        the link, not the target.
+        """
+        link = self.path.child("sub1.link")
+        # setUp creates the sub1 child
+        os.symlink(self.path.child("sub1").path, link.path)
+        link.remove()
+        self.assertFalse(link.exists())
+        self.assertTrue(self.path.child("sub1").exists())
+
+    if getattr(os, 'symlink', None) is None:
+        test_removeWithSymlink.skip = "Platform doesn't support symbolic links"
+
+
     def testCopyTo(self):
         self.assertRaises((OSError, IOError), self.path.copyTo, self.path.child('file1'))
         oldPaths = list(self.path.walk()) # Record initial state
